import logging
from binascii import hexlify

import zmq.asyncio
from loraapi import control_rpc_pb2
from loraapi import control_statistic_pb2

log = logging.getLogger('lorad')


def assemble_stats(rt):
    #  Instantiate control rpc response
    reply = control_rpc_pb2.Reply()
    reply.status = reply.OK

    #  Initialize statistics field
    s_reply = control_statistic_pb2.Reply()

    #  Add router stats
    rtStats = rt.getStats()
    for name, value in rtStats.items():
        stat = s_reply.Statistic()
        stat.name = name
        if type(value) is not list:
            stat.unsigned_int = value
        else:
            stat.string = str(value).upper()  # .replace("'","")
        s_reply.statistics.append(stat)

    #  Convert stats structure to bytes
    reply.protobuf = s_reply.SerializeToString()

    return reply


class Stats():
    def __init__(self):
        ctx = zmq.asyncio.Context()
        sockctrl = ctx.socket(zmq.REP)
        sockctrl.bind("ipc:///var/run/lora/station")
        self.sockctrl = sockctrl
        self.router = None

    async def recv_task(self, rt):
        self.router = rt
        while True:
            message = await self.sockctrl.recv()
            log.debug("Stats: received message " +
                      hexlify(message).decode("ascii"))
            request = control_rpc_pb2.Request()
            request.ParseFromString(message)
            log.debug("Stats: translated message " + str(request))

            if "get_statistics" == request.service:
                reply = assemble_stats(rt)
                self.sockctrl.send(reply.SerializeToString())
